var db           = require('diskdb');
var _            = require("lodash");
var moment       = require("moment");
var EventEmitter = require("events").EventEmitter;
var util         = require("util");
var db_path      = "db";

function model (name) {
	this.name = name;
	this.db = db.connect(db_path, [this.name]);
	EventEmitter.call(this);
}
util.inherits(model, EventEmitter);
model.prototype
	.findOne = function (data) {
			return this.db[this.name].findOne(data) || {};
		};
model.prototype
	.select = function (query) {
		var results = []; // Set results function scope variable

			// If the query is empty after we run remove oderBy, run the find method with empty args.
			if(_.isEmpty(_.omit(query, "orderBy"))){
				results = this.db[this.name].find();

				// If oderBy exist is query, sort by the orderBy key, otherwise, return the orignal results.
				if(_.has(query, "orderBy")){
					return _.sortBy(results, query.orderBy);
				}else{
					return results;
				}
			}else{

				// Run the query with the orderBy key removed.
				results = this.db[this.name].find(_.omit(query, "orderBy"));
				if(_.has(query, "orderBy")){
					return _.sortBy(results, "email");
				}else{
					return results;;
				}
			}
		};
model.prototype
	.insert = function (data) {
		var date = moment();
		data.created = date;
		data.updated = date;
		var results = this.db[this.name].save(data);
		if (results){
			this.emit(this.name, 
			{
				method: "insert",
				old_value: null,
				new_value: results
			});
			return results
		}
		else{
			return {};
		}	
	};
model.prototype
	.update = function (_id, data) {
		var old_data = this.findOne({_id: _id});
		data.updated = moment();
		var results = this.db[this.name].update({_id: _id}, data);
		var new_data = this.findOne({_id: _id});
			if(results.updated >= 1){
				this.emit(this.name, 
					{
						method: "update",
						old_value: old_data,
						new_value: new_data
					});
			}
		};
model.prototype
	.delete = function (data) {
			this.emit(this.name, 
				{
					method:"delete",
					old_value:data,
					new_value:null
				});
			return this.db[this.name].remove(data, false);
		};

module.exports = model;