var _ = require("lodash");
var express = require("express");

var model = require("./model");
var User = require("./user");
var jwt = require('jwt-simple');
var config = require("./config");
var moment = require("moment");

module.exports = function (module_config, io) {
	if(module_config){
		config = module_config;
	}
	if (io){
		io.on("connection", function (socket) {
			socket.on("new", function (data) {
				console.log(data);
			});
			socket.emit("new", {message:"THis is the 'new' event"})
		});
	}

	return {
		collection: function (collection_name) {
			var collection = new model(collection_name);

			// Listen for modification events on the collection
			// This is where we pass modification data to socket.io connections
			collection.on(collection.name, function (data) {
				console.log(collection.name, data);
			});

			// Create http RESTful Routes for collection.
			var collection_router = express.Router();
			collection_router
				.get('/:_id?', function(req, res) {
					console.log("collection_name");
					if (req.params._id){
						res.json(collection.findOne({_id: req.params._id}));
					} else {
						res.json(collection.select(req.query));
					}

				})
				.post('/', function(req, res) {
					if(req.body){
						var new_user = req.body;
						res.json(collection.insert(new_user));
					}else{
						res.status(400).json(
							{
								status:false,
								message:"No user sent with request"
							});
					}
				})
				.put('/:_id?', function(req, res) {
					if(req.body && (req.body._id || req.params._id)){
						var _id = req.body._id || req.params._id;
						var updating_user = _.omit(req.body, _id);
						res.json(collection.update(_id, updating_user));
					}else{
						res.status(400).json(
							{
								status:false,
								message:"No user sent with request"
							});
					}
				})
				.delete('/:_id?', function(req, res) {
					if (req.body._id || req.params._id) {
						var _id = req.body._id || req.params._id;
						res.json(collection.delete({_id: _id}));
					}else{
						res.status(400).json(
							{
								status:false,
								message:"No user id sent with request"
							});
					}
				});
			return collection_router;
		},
		user: function (user_config) {
			var collection = new User();

			// Listen for modification events on the collection
			// This is where we pass modification data to socket.io connections
			collection.on(collection.name, function (data) {
				console.log(collection.name, data);
			});
			var user_router = express.Router();
			user_router
				.get('/:_id?', function(req, res) {
					console.log("collection_name");
					if (req.params._id){
						res.json(collection.findOne({_id: req.params._id}));
					} else {
						res.json(collection.select(req.query));
					}

				})
				.post('/', function(req, res) {
					if(req.body){
						var new_user = req.body;
						var results = collection.insert(new_user);
						if(results){
							res.json(results);
						}else{
							res.status(400).json({status:400, message:"Username or Email already exist!"});
						}

					}else{
						res.status(400).json(
							{
								status:false,
								message:"No user sent with request"
							});
					}
				})
				.put('/:_id?', function(req, res) {
					if(req.body && (req.body._id || req.params._id)){
						var _id = req.body._id || req.params._id;
						var updating_user = _.omit(req.body, _id);
						res.json(collection.update(_id, updating_user));
					}else{
						res.status(400).json(
							{
								status:false,
								message:"No user sent with request"
							});
					}
				})
				.delete('/:_id?', function(req, res) {
					if (req.body._id || req.params._id) {
						var _id = req.body._id || req.params._id;
						res.json(collection.delete({_id: _id}));
					}else{
						res.status(400).json(
							{
								status:false,
								message:"No user id sent with request"
							});
					}
				})
				.post("/login", function (req, res) {
					var results = collection.authenticate(req.body);
					var attempts = new model("attempts");
					var current_date = moment();
					// Deals with json web tokens if they are activated
					if(config.jwt && results.status == 200){
						var expires = moment().add('days', 7).valueOf();

						var payload = {
								user:results.user,
								expires:expires
							};
						var access_token = jwt.encode(payload, config.secret_key);
						var jwt_collection = new model("jwt");
						var new_jwt = {
								user:results.user._id,
								token:access_token,
								date:current_date,
								expires:expires
							};
						jwt_collection.insert(new_jwt);
						results[config.token_key] = access_token;
					}
					if(results.status == 200){
						var new_attempt = {
							date:current_date,
							user:req.body,
							allowed:true
						};
						attempts.insert(new_attempt);
					}else{
						var new_attempt = {
							date:current_date,
							user:req.body,
							allowed:false
						};
						attempts.insert(new_attempt);
					}
					res.status(results.status).json(results);
				})
				.post("/logout", function (req, res) {
					// var results = collection.authenticate(req.body);
					// res.status(results.status).json(results);
					res.json({status:200,message:"user logged out"});
				})
				.post("/validate", function (req, res) {
					var payload = null;
					try{
						payload = jwt.decode(req.headers.access_token, config.secret_key);
					} catch(err){
						res.status(401).json({
							status:false,
							message:"Invalid Token"
						});
					}

					if(payload){
						if(payload.expires <= Date.now()){
							res.status(401).json({
								status:false,
								message:"Invalid Token"
							});
						}
						else {
							console.log("got here");
							res.status(200).json({
								user:payload.user
							});
						}

					}else{

						res.status(401).json({
							status:false,
							message:"Invalid Token"
						});
					}
					// var results = collection.authenticate(req.body);
					// res.status(results.status).json(results);
				});
			user_router.collection = collection;
			return user_router;
		},
		model:function (name) {
			 return new model(name) ;
		},
		user_model:function (name) {
			 return new User() ;
		}
	};
}
